#include <SDL.h>
#include <cstdlib> // atexit
#include <iostream>
#include <memory> // std::unique_ptr

#include "sdl_helper.h"

// WICHTIG:
// SDL2 unter Windows benoetigt unbedingt main(int, char**), sonst gibt es einen Fehler
// Die Variablennamen für die Parameter wurden weggelassen, sonst gibt es Compilerwarnungen
// (ungenutzter Parameter)
int main(int, char**) {
    /* SDL initialisieren */
    const int width = 1024;
    const int height = 768;

    // Alle Komponenten von SDL initialisieren
    // siehe https://wiki.libsdl.org/SDL_Init
    if (SDL_Init(SDL_INIT_EVERYTHING) < 0) {
        std::cerr << "SDL_Init fehlgeschlagen " << SDL_GetError() << "\n";
        return 1;
    }

    // Beim beenden SDL_Quit aufrufen (Resourcen freigeben)
    atexit(SDL_Quit);

    // Fenster erstellen, siehe https://wiki.libsdl.org/SDL_CreateWindow
    // Parameter:
    // - Text in Titelleiste
    // - X & Y Position (Zahl in Pixeln oder Konstante)
    // - Fensterbreite & -hoehe in Pixeln
    // - Flags für den Fenstermanager
    std::unique_ptr<SDL_Window, SdlDeleter> window(SDL_CreateWindow(
        "Praktikum11 SDL",
        SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
        width, height,
        SDL_WINDOW_RESIZABLE));

    if (!window) {
        std::cerr << "SDL_CreateWindow fehlgeschlagen " << SDL_GetError() << "\n";
        return 1;
    }

    // Renderer erstellen, siehe https://wiki.libsdl.org/SDL_CreateRenderer
    // Ein Renderer wird zum Zeichnen in ein Fenster verwendet
    // Parameter:
    // - SDL_Window handle von oben
    // - Zu verwendenter Treiber (-1 = Den ersten funktionierenden verwenden)
    // - Flags: Keine Notwendig
    std::unique_ptr<SDL_Renderer, SdlDeleter> renderer(
        SDL_CreateRenderer(window.get(), -1, 0));

    if (!renderer) {
        std::cerr << "SDL_CreateRenderer fehlgeschlagen " << SDL_GetError() << "\n";
        return 1;
    }

    // Automatisch skalieren wenn sich Fenstergroesse aendert
	SDL_RenderSetLogicalSize(renderer.get(), width, height);

    // Fenster schwarz ausmalen
	SDL_RenderClear(renderer.get());

    // Aenderung sichtbar machen
	SDL_RenderPresent(renderer.get());

    /* Event Loop */

    for (;;) {
        // Nutzereingaben verarbeiten
        SDL_Event evt;

        // Events einlesen
        // siehe https://wiki.libsdl.org/SDL_Event
        while (SDL_PollEvent(&evt)) {
            // Jeweils ein Event verarbeiten
            if (evt.type == SDL_QUIT) {
                // Benutzer hat Programmende angefordert
                return 0;
            }
        }

        /* Updatelogik */
        // Rechteck definiert mit X, Y, Breite (w) und Höhe (h)
        SDL_Rect rect = { 300, 300, 200, 200 };

        /* Zeichnen / Rendern */
        // Fenster schwarz ausmalen
        SDL_SetRenderDrawColor(renderer.get(), 0xFF, 0xFF, 0xFF, 0xFF);
        SDL_RenderClear(renderer.get());

        // Zeichenfarbe auf rot setzen
        SDL_SetRenderDrawColor(renderer.get(), 0xFF, 0x0, 0x0, 0xFF);

        // Rechteck zeichnen
        SDL_RenderFillRect(renderer.get(), &rect);

        // Aenderung sichtbar machen
        SDL_RenderPresent(renderer.get());
    }

    return 0;
}
